# FILE: local.tcl
#
# Configuration variables for the hosts in NIST/ITL/MCSD and NIST/MSEL/MD
#

if {![info exists tcl_platform]} {
# Assume Tcl 7.4 interpreters are running on Linux/Intel machines
    Default PLATFORM	"unix"
    Default MACHINE	"i386"
    Default OS		"linux"
    Default OSVERSION	"2.0.0"
} else {
# Need these values to continue, so get the defaults.
    Default MACHINE [string tolower [string trim $tcl_platform(machine)]]
    Default PLATFORM [string tolower [string trim $tcl_platform(platform)]]
    Default OS [string tolower [string trim $tcl_platform(os)]]
    Default OSVERSION [string tolower [string trim $tcl_platform(osVersion)]]
}
# Need these values to continue, so get the defaults.
# set OSMAJOR $OSVERSION
regexp {([^.]*).*} $OSVERSION match OSMAJOR

# Construct output directory name
if [string match i*86 $MACHINE] {
    set BINDIR bin-i386
} elseif [string match sun* $MACHINE] {
    set BINDIR bin-sun
} elseif [string match ip* $MACHINE] {
    set BINDIR bin-sgi
} else {
    set BINDIR bin-$MACHINE
}

if { [string compare $OS irix64]==0 } {
    # 64-bit SGI
# Tcl 7.4/Tk 4.0
#    Default TCLINCLUDEDIRECTORY	"/usr/local/src/tcl/include"
#    Default TKINCLUDEDIRECTORY		"/usr/local/src/tcl/include"
#    Default TCLLIBRARYDIRECTORY	"/usr/local/src/tcl/tcl7.4"
#    Default TKLIBRARYDIRECTORY		"/usr/local/src/tcl/tk4.0"
#    Default X11LIBRARYDIRECTORY	"/usr/X11/lib"
#    Default TCLLIBRARYFILE		"libtcl.a"
#    Default TKLIBRARYFILE		"libtk.a"
# Tcl 7.6/Tk 4.2
#    Default TCLINCLUDEDIRECTORY	"/usr/local/apps/nisttcl/include"
#    Default TKINCLUDEDIRECTORY		"/usr/local/apps/nisttcl/include/tk4.2"
#    Default TCLLIBRARYDIRECTORY		"/usr/local/apps/nisttcl/lib"
#    Default TKLIBRARYDIRECTORY		"/usr/local/apps/nisttcl/lib"
#    Default X11LIBRARYDIRECTORY		"/usr/X11/lib"
#    Default TCLLIBRARYFILE		"libtcl7.6.a"
#    Default TKLIBRARYFILE		"libtk4.2.a"
# Tcl 8.0/Tk 8.0
    Default TCLINCLUDEDIRECTORY		"/itl/links/generic/include"
    Default TKINCLUDEDIRECTORY		"/itl/links/generic/include"
    Default TCLLIBRARYDIRECTORY		"/itl/links/generic/lib"
    Default TKLIBRARYDIRECTORY		"/itl/links/generic/lib"
    Default X11LIBRARYDIRECTORY		"/usr/X11/lib"
    Default TCLLIBRARYFILE		"libtcl8.0.so"
    Default TKLIBRARYFILE		"libtk8.0.so"
} elseif { [string compare $OS sunos]==0 && $OSMAJOR<5} {
    # SunOS
#    Default TCLINCLUDEDIRECTORY		"/cam/expect96/include"
#    Default TKINCLUDEDIRECTORY		"/cam/expect96/include"
#    Default X11INCLUDEDIRECTORY		"/usr/openwin/include"
#    Default TCLLIBRARYDIRECTORY		"/usr/local/lib"
#    Default TKLIBRARYDIRECTORY		"/usr/local/lib"
#    Default X11LIBRARYDIRECTORY		"/usr/openwin/lib"
#    Default TCLLIBRARYFILE		"libtcl76.so"
#    Default TKLIBRARYFILE		"libtk42.so"
    Default TCLINCLUDEDIRECTORY		"/itl/links/generic/include"
    Default TKINCLUDEDIRECTORY		"/itl/links/generic/include"
    Default X11INCLUDEDIRECTORY		"/usr/openwin/include"
    Default TCLLIBRARYDIRECTORY		"/itl/links/generic/lib"
    Default TKLIBRARYDIRECTORY		"/itl/links/generic/lib"
    Default X11LIBRARYDIRECTORY		"/usr/openwin/lib"
    Default TCLLIBRARYFILE		"libtcl80.so"
    Default TKLIBRARYFILE		"libtk80.so"
} elseif { [string compare $OS sunos]==0 && $OSMAJOR>=5} {
    # Solaris
    Default TCLINCLUDEDIRECTORY		"/itl/links/generic/include"
    Default TKINCLUDEDIRECTORY		"/itl/links/generic/include"
    Default X11INCLUDEDIRECTORY		"/usr/openwin/include"
    Default TCLLIBRARYDIRECTORY		"/itl/links/generic/lib"
    Default TKLIBRARYDIRECTORY		"/itl/links/generic/lib"
    Default X11LIBRARYDIRECTORY		"/usr/openwin/lib"
    Default TCLLIBRARYFILE		"libtcl8.0.so"
    Default TKLIBRARYFILE		"libtk8.0.so"
#    Default TCLINCLUDEDIRECTORY		"/cam/expect96/include"
#    Default TKINCLUDEDIRECTORY		"/cam/expect96/include"
#    Default X11INCLUDEDIRECTORY		"/usr/openwin/include"
#    Default TCLLIBRARYDIRECTORY		"/usr/local/lib"
#    Default TKLIBRARYDIRECTORY		"/usr/local/lib"
#    Default X11LIBRARYDIRECTORY		"/usr/openwin/lib"
#    Default TCLLIBRARYFILE		"libtcl7.6.so"
#    Default TKLIBRARYFILE		"libtk4.2.so"
#    Default TCLINCLUDEDIRECTORY		"/home/fs3c/dgp/include"
#    Default TKINCLUDEDIRECTORY		"/home/fs3c/dgp/include"
#    Default X11INCLUDEDIRECTORY		"/usr/openwin/include"
#    Default TCLLIBRARYDIRECTORY		"/home/fs3c/dgp/lib"
#    Default TKLIBRARYDIRECTORY		"/home/fs3c/dgp/lib"
#    Default X11LIBRARYDIRECTORY		"/usr/openwin/lib"
#    Default TCLLIBRARYFILE		"libtcl8.0.so"
#    Default TKLIBRARYFILE		"libtk8.0.so" 
} elseif { [string compare $OS linux]==0 \
        && [string match {i?86} $MACHINE]==1 } {
    # Linux/Intel
    Default TCLINCLUDEDIRECTORY		"/usr/local/include"
    Default TKINCLUDEDIRECTORY		"/usr/local/include"
    Default X11LIBRARYDIRECTORY		"/usr/X11/lib"
#    Default TCLLIBRARYFILE		"libtcl.a"
#    Default TKLIBRARYFILE		"libtk.a"
    Default TCLLIBRARYFILE		"libtcl.so"
    Default TKLIBRARYFILE		"libtk.so"
    Default DLLIBRARYFILE		"libdl.so"
} elseif { [string compare $OS linux]==0 \
	&& [string compare $MACHINE alpha]==0 } {
    # Linux/AXP
# Tcl 7.4/Tk 4.0
#    Default TCLINCLUDEDIRECTORY	"/usr/include/tcl"
#    Default TKINCLUDEDIRECTORY		"/usr/include/tcl"
#    Default X11LIBRARYDIRECTORY	"/usr/X11/lib"
#    Default TCLLIBRARYFILE		"libtcl.a"
#    Default TKLIBRARYFILE		"libtk.a"
# Tcl 7.6/Tk 4.2
    Default TCLINCLUDEDIRECTORY		"/usr/local/include"
    Default TKINCLUDEDIRECTORY		"/usr/local/include"
    Default X11LIBRARYDIRECTORY		"/usr/X11/lib"
#    Default TCLLIBRARYFILE		"libtcl7.6.a"
#    Default TKLIBRARYFILE		"libtk4.2.a"
#    Default TCLLIBRARYFILE		"libtcl.a"
#    Default TKLIBRARYFILE		"libtk.a"
    Default TCLLIBRARYFILE		"libtcl.so"
    Default TKLIBRARYFILE		"libtk.so"
} elseif { [string compare $PLATFORM windows]==0 \
	&& [string compare $MACHINE intel]==0} { ;# Window/intel
    # Assume Windows on cascade
    if {$tcl_version < 7.5} {
      puts "Error: You're running build.tcl with Tcl $tcl_version"
      puts "	build.tcl requires Tcl 7.5 or later."
      exit 1
    } elseif {$tcl_version < 7.6} {
      Default TCLLIBRARYFILE		"tcl75.lib"
      Default TKLIBRARYFILE		"tk41.lib"
    } elseif {$tcl_version < 8} {
      Default TCLLIBRARYFILE		"tcl76.lib"
      Default TKLIBRARYFILE		"tk42.lib"
    } else {
      Default TCLLIBRARYFILE		"tcl80vc.lib"
      Default TKLIBRARYFILE		"tk80vc.lib"
    }
    if {[string match "windows nt" $OS]} {
#      Default TCLINCLUDEDIRECTORY	"/Program Files/Tcl/include"
#      Default TKINCLUDEDIRECTORY	"/Program Files/Tcl/include"
#      Default X11INCLUDEDIRECTORY	"/Program Files/Tcl/include"
#      Default TCLLIBRARYDIRECTORY	"/Program Files/Tcl/lib"
#      Default TKLIBRARYDIRECTORY	"/Program Files/Tcl/lib"
      Default TCLINCLUDEDIRECTORY	"/Tcl/include"
      Default TKINCLUDEDIRECTORY	"/Tcl/include"
      Default X11INCLUDEDIRECTORY	"/Tcl/include"
      Default TCLLIBRARYDIRECTORY	"/Tcl/lib"
      Default TKLIBRARYDIRECTORY	"/Tcl/lib"
    } else {
      Default TCLINCLUDEDIRECTORY	"/Program Files/Tcl/include"
      Default TKINCLUDEDIRECTORY	"/Program Files/Tcl/include"
      Default X11INCLUDEDIRECTORY	"/Program Files/Tcl/include"
      Default TCLLIBRARYDIRECTORY	"/Program Files/Tcl/lib"
      Default TKLIBRARYDIRECTORY	"/Program Files/Tcl/lib"
    }
    Default INSTALLROOT			"/oommf"
    Default SOURCEDISTRIBUTIONDIRECTORY "/users/dgp/oommfsrc"
}

# Quick hack to set an installation directory for DGP
if [info exists env(USER)] {
  if {[string compare $env(USER) dgp]==0} {
    Default INSTALLROOT $env(HOME)
    Default SOURCEDISTRIBUTIONDIRECTORY $env(HOME)/distrib
  }
  if {[string compare $env(USER) dgporter]==0} {
    Default INSTALLROOT /dos/oommf
    Default SOURCEDISTRIBUTIONDIRECTORY /dos/distrib
  }
}


