# FILE: mmSolve2D.tcl
#
# This file must be interpreted by the mmsolve shell.

# Support libraries
package require net

set commands {
    Solver	{{LoadProblem RemoteRelayInput ProblemDescription}}
}

array set inputs {
    ProblemDescription	{ProbEd
			 GetProbDescFile
			 LoadProbDescFile
                        }
}
array set events {}
array set outputs {}

proc LoadProbDescFile { file } {
    global commands events inputs outputs
    catch { unset inputs }
    catch { unset outputs }  ;# "outputs" is _not_ always set!
    catch { unset events }
    catch { unset commands } 
    array set inputs {}
# The commands which can be invoked remotely
set commands {
    Solver	{
		 {Reset ResetThread} 
                 {Run ChangeSolverState Run}  
                 {Relax ChangeSolverState Relax}  
                 {Pause ChangeSolverState Pause}  
		 {Field- FieldAdjust -1}
		 {Field+ FieldAdjust 1}
		}
}

# The events generated by this thread.
array set events {
    Iteration	{1 solver GetODEStepCount}
    FieldStep	{1 solver GetFieldStepCount}
}

# The quantities which this thread produces as output
#	The last entry should be moved to be set by controller
array set outputs {
    Magnetization	{display
			 {Iteration FieldStep}
			 solver
			 WriteMagFile
			 openfile
                        }
    TotalField 		{display
			 {Iteration FieldStep}
			 solver
			 WriteFieldFile
			 openfile
                        }
    Label		{log
			 {Iteration FieldStep}
			 solver
			 GetIterationValueLabels
			 Append
                        }
    IterationValues	{log
			 {Iteration FieldStep}
			 solver
			 GetIterationValues
			 Append
                        }
    DataTable		{DataTable
			 {Iteration FieldStep}
			 ""
			 MakeDataTable
			 DataTable
			}
}


    # Overkill until something better is designed
    foreach c [net_connection instances] {
        $c Tell InterfaceChange
    }
    global solver protocol
    set solver [omf_solver new -mif_filename $file]
    $protocol Wrap $solver 1
    PrepSolver
    ResetThread
    em_eventHandler new $solver delete { after 2000 exit }
}

set protocol [net_protocol new -name "OOMMF solver protocol 0.0"]
$protocol AddMessage start HasGui {} {
    return [list start [list 0 1]]
}
$protocol AddMessage start Commands {} {
    global commands
    return [list start [list 0 $commands]]
}
$protocol AddMessage start Outputs {} {
    global outputs
    return [list start [list 0 [array get outputs]]]
}
$protocol AddMessage start Inputs {} {
    global inputs
    return [list start [list 0 [array get inputs]]]
}
$protocol AddMessage start Events {} {
    global events
    return [list start [list 0 [array get events]]]
}
$protocol AddMessage start SetHandler {args} {
    return [list start [list 0 [eval SetHandler $args]]]
}
$protocol AddMessage start SetInput {args} {
    return [list start [list 0 [eval SetInput $args]]]
}
$protocol AddMessage start UnsetHandler {args} {
    return [list start [list 0 [eval UnsetHandler $args]]]
}
$protocol AddMessage start RemoteRelayOutput {args} {
    return [list start [list 0 [eval RemoteRelayOutput $args]]]
}
$protocol AddMessage start RemoteRelayInput {args} {
    return [list start [list 0 [eval RemoteRelayInput $args]]]
}
$protocol AddMessage start FieldAdjust {args} {
    return [list start [list 0 [eval FieldAdjust $args]]]
}
$protocol AddMessage start ResetThread {args} {
    return [list start [list 0 [eval ResetThread $args]]]
}
$protocol AddMessage start ChangeSolverState {args} {
    return [list start [list 0 [eval ChangeSolverState $args]]]
}
$protocol AddMessage start delete {} {
    global protocol
    return [list start [list 0 [after 100 $protocol delete; after 2000 exit]]]
}
set server [net_server new -protocol $protocol -alias mmSolve2D \
        -log [em_extension ApplicationLog]]
$server Start 0

proc SetInput {data host acct pid} {
    global sourcethread
    set t [net_thread new -hostname $host -accountname $acct -pid $pid \
            -alias {} -log [em_extension ApplicationLog]]
    set sourcethread($data) $t
}
proc SetHandler {event data host acct pid cnt} {
    global events
    UnsetHandler $event $data $host $acct $pid
    global handler outputs
    set t [net_thread new -hostname $host -accountname $acct -pid $pid \
            -alias {} -log [em_extension ApplicationLog]]
    set evinfo $events($event)
    set srcName [lindex $evinfo 1]
    upvar #0 $srcName src
    set handler($event,$host,$acct,$pid,$data) [em_eventHandler new $src \
            $event [list GenericHandler $event $data $cnt $t]]
}

proc GenericHandler {event data cnt thread} {
    global events
    set evinfo $events($event)
    if {[lindex $evinfo 0]} {
        set srcName [lindex $evinfo 1]
        upvar #0 $srcName src
        set evcount [eval $src [lindex $evinfo 2]]
        if {$evcount % $cnt} {
            return
        }
    }
    RelayOutput $data $thread
}

proc RemoteRelayOutput { data host acct pid } {
    set t [net_thread new -hostname $host -accountname $acct -pid $pid \
            -alias {} -log [em_extension ApplicationLog]]
    RelayOutput $data $t
}

proc RelayOutput { data thread } {
    global outputs
    catch {after cancel RelayOutput $data $thread}
    set opinfo $outputs($data)
    set srcName [lindex $opinfo 2]
    if {[string length $srcName]} {
        upvar #0 $srcName src
        set datavalue [eval $src [lindex $opinfo 3]]
    } else {
        set datavalue [eval [lindex $opinfo 3]]
    }
    set method [lindex $opinfo 4]
    if {[catch {$thread Ready} ready] || !$ready} {
        after 20 RelayOutput $data $thread
    } else {
        $thread Send $method $datavalue
    }
}

proc RemoteRelayInput { data } {
    global sourcethread
    RelayInput $data $sourcethread($data)
}

proc RelayInput { data thread } {
    global inputs handler
    catch {after cancel RelayInput $data $thread}
    if {[catch {$thread Ready} ready] || !$ready} {
        after 100 RelayInput $data $thread
    } else {
        set ininfo $inputs($data)
        set qid [$thread Send [lindex $ininfo 1]]    
        set handler(input,[$thread cget -hostname],[$thread cget -accountname],[$thread cget -pid],$data) \
               [em_eventHandler new $thread Reply$qid \
               [list ReceiveRelayInputReply $thread $data]]
    }
}

proc ReceiveRelayInputReply {thread data} {
    global handler inputs
    set host [$thread cget -hostname]
    set acct [$thread cget -accountname]
    set pid [$thread cget -pid]
    $handler(input,$host,$acct,$pid,$data) delete
    unset handler(input,$host,$acct,$pid,$data)
    set reply [$thread Get]
    if {![lindex $reply 0]} {
        set ininfo $inputs($data)
        eval [lindex $ininfo 2] [lrange $reply 1 end]
    }
}
proc UnsetHandler {event data host acct pid} {
    global handler
    if {[info exists handler($event,$host,$acct,$pid,$data)]} {
        $handler($event,$host,$acct,$pid,$data) delete
        unset handler($event,$host,$acct,$pid,$data)
    }
}

# Initialize solver
#$solver configure -relax_stop_torque 1e-5
set Bfield_multiplier 1000  ;# Convert T to mT

#if {$argc>3} { set Bdir_nom [lrange $argv 1 3] }
#if {$argc>4} { set Bdir_label [lindex $argv 4] } ;# Label to put on M vs. B loop

## If Bdir_nom (nominal applied field direction) is left unset
## (i.e., not specified on the command line), then the code
## assumes the nominal direction is along one of the coordinate
## axis, and tries to extract it from the MIF file.

proc Dot { vec1 vec2 } {
    # Dot product of two 3-lists
    return    [expr [lindex $vec1 0]*[lindex $vec2 0] \
	    +       [lindex $vec1 1]*[lindex $vec2 1] \
	    +       [lindex $vec1 2]*[lindex $vec2 2]]
}

proc PrepSolver {} {
global solver Bdir_nom Bdir_label
# Guess Bdir_nom, if necessary
if {![info exists Bdir_nom]} {
    set Bdir_nom [$solver GuessAppliedFieldAxis]
}
if {![info exists Bdir_label]} {
    if {[Dot $Bdir_nom "1 0 0"] > .9} {
	set Bdir_label "B_x applied (mT)"
    } elseif {[Dot $Bdir_nom "0 1 0"] > .9} {
	set Bdir_label "B_y applied (mT)"
    } elseif {[Dot $Bdir_nom "0 0 1"] > .9} {
	set Bdir_label "B_z applied (mT)"
    } else {
	set Bdir_label "B applied (mT)"
    }
}
}

proc MakeDataTable {} {
    global solver Bfield_multiplier Bdir_nom
    set table [list [list Iteration "" [$solver GetODEStepCount]]]
    set Bfield [$solver GetNomAppliedField]
    lappend table [list Bx mT [expr $Bfield_multiplier * [lindex $Bfield 0]]]
    lappend table [list By mT [expr $Bfield_multiplier * [lindex $Bfield 1]]]
    lappend table [list Bz mT [expr $Bfield_multiplier * [lindex $Bfield 2]]]
    lappend table [list B mT [expr $Bfield_multiplier * \
            [Dot $Bdir_nom $Bfield]]]
    lappend table [list Torque "" [$solver GetMaxTorque]]
    set mag [$solver GetAveMag]
    lappend table [list Mx/Ms "" [lindex $mag 0]]
    lappend table [list My/Ms "" [lindex $mag 1]]
    lappend table [list Mz/Ms "" [lindex $mag 2]]
    set energies [join [$solver GetEnergies]]
    lappend table [list "Total\nEnergy"      "" [lindex $energies 1]]
    lappend table [list "Exchange\nEnergy"   "" [lindex $energies 3]]
    lappend table [list "Anisotropy\nEnergy" "" [lindex $energies 5]]
    lappend table [list "Demag\nEnergy"      "" [lindex $energies 7]]
    lappend table [list "Zeeman\nEnergy"     "" [lindex $energies 9]]
    lappend table [list "Max\nAngle" deg [$solver GetMaxAngle]]
    return $table
}

proc InspectLoop { solver } {
    after cancel InspectLoop $solver ;# There can be only 1. ;-/
    global last_step
    set this_step [$solver GetODEStepCount]
    if {![info exists last_step] || $last_step!=$this_step} {
	if {1 || $this_step % 10 == 0} {
	    set torque [$solver GetMaxTorque]
	    puts "Step [format "%4d" $this_step]: \
		    Max Torque=[format "%7.5f" $torque]"
	}
    }
    set last_step $this_step
    update idletasks  ;# Let the above output requests be
    ## handled before putting InspectLoop back into the
    ## event stack (otherwise if it takes too long to service
    ## the aforementioned requests, the new InspectLoop request
    ## can become ripe before they are completed, and we reenter
    ## this proc without servicing background processes.
    after 20 InspectLoop $solver
}

proc FieldAdjust { step } {
    global solver 
    if {$step != 0} {
	$solver IncrementFieldStep $step
    }
}

proc ResetThread {} {
    global solver
    ChangeSolverState Pause
    $solver Reset
    FieldAdjust 0
}

proc ChangeSolverState { newstate } {
    # NOTE: Restarting the ODE solver after a pause resets the
    #  internal solver state, i.e., you lose previous magnetization
    #  and field state info used in by the predictor-corrector
    #  algorithm.  As a result, interactive pausing may cause the
    #  solver results to be *slightly* different.
    global solver
    # Reset button displays

    # Go to new state
    if { [string match "Run" $newstate] } {
	update
	InspectLoop $solver
	$solver HystLoop "UpdateGraphs bapp" "FieldAdjust 0"
    } elseif { [string match "Relax" $newstate] } {
	update
	InspectLoop $solver
	$solver RelaxState "UpdateGraphs bapp"
    } else {
	# Request solver pause
	$solver configure -stop_request_flag USER
    }

    # Upon return from the HystLoop or RelaxState calls,
    # or in the event of a Pause solver request, go
    # into pause mode
    InspectLoop $solver ;# Process any pending output
    after cancel InspectLoop $solver ;# No mas!
    update
}

proc bgerror { msg } {
    [em_extension ApplicationLog] Append $msg
}
vwait omf_forever
